"use strict";

let cv, gl;

let program;

let vertices;
let vertexBuffer;

let lastFrame = Date.now();

async function init() {
	cv = document.getElementById("cv");
	gl = cv.getContext("webgl");

	if (!gl) {
		window.alert("WebGL not supported");
	}

	console.log("compiling shaders...");

	console.log("compiling vertex shader...");
	let vertexShader = await fetchAndCompileShader(gl, gl.VERTEX_SHADER, "shaders/vertex.glsl");

	console.log("compiling fragment shader...");
	let fragmentShader = await fetchAndCompileShader(gl, gl.FRAGMENT_SHADER, "shaders/fragment.glsl");

	console.log("linking shader program...");
	program = linkShaderProgram(gl, vertexShader, fragmentShader);

	/**
	 * -0.35              0.35
	 * |     -0.2    0.2  |
	 * |    |        |    |
	 * 
	 * +----+        +----+    ---  0.6
	 * |    |        |    |    
	 * |    |        |    |
	 * |    +--------+    |    ---  0.1
	 * |                  |
	 * |    +--------+    |    --- -0.1
	 * |    |        |    |
	 * |    |        |    |
	 * +----+        +----+    --- -0.6
	 *
	 * +------------------+    --- -0.7
	 * |                  |
	 * +------------------+    --- -0.9
	 *   
	 */    

	vertices = [
		// X  // Y

		// left vertical bar
		-0.35, -0.6,
		-0.35, 0.6,
		-0.2, 0.6,

		-0.35, -0.6,
		-0.2, 0.6,
		-0.2, -0.6,

		// right vertical bar
		0.35, -0.6,
		0.35, 0.6,
		0.2, 0.6,

		0.35, -0.6,
		0.2, 0.6,
		0.2, -0.6,

		// middle bar
		-0.2, -0.1,
		-0.2, 0.1,
		0.2, 0.1,

		-0.2, -0.1,
		0.2, 0.1,
		0.2, -0.1,

		// bottom bar
		-0.35, -0.7,
		-0.35, -0.9,
		0.35, -0.9,

		-0.35, -0.7,
		0.35, -0.9,
		0.35, -0.7
	];

	console.log("creating vertex buffer");

	vertexBuffer = gl.createBuffer();
	gl.bindBuffer(gl.ARRAY_BUFFER, vertexBuffer);
	gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(vertices), gl.STATIC_DRAW);

	// get location of vertPosition attribute
	let positionAttribLocation = gl.getAttribLocation(program, "vertPosition");

	gl.vertexAttribPointer(
		positionAttribLocation, // which attribute is read
		2, // number of values to read
		gl.FLOAT, // type of values
		gl.FALSE, // whether to normalize
		2 * Float32Array.BYTES_PER_ELEMENT, // size of individual vertex / distance between values to read
		0 // offset where to start reading
	);

	// enable attribute
	gl.enableVertexAttribArray(positionAttribLocation);

	// unbind buffer
	gl.bindBuffer(gl.ARRAY_BUFFER, null);

	// set clear colour
	gl.clearColor(0.1, 0.1, 0.1, 1.0);

	// start drawing
	requestAnimationFrame(draw);
}

let frameCount = 0;
let lastStatUpdate = Date.now();

function updateStats(deltaTime) {
	frameCount++;

	let now = Date.now();
	let timeSinceLastUpdate = (now - lastStatUpdate) / 1000;

	if (timeSinceLastUpdate < 0.5) return;

	lastStatUpdate = now;

	let fpsDisplay = document.getElementById("fps");
	fpsDisplay.innerText = Math.round(frameCount / timeSinceLastUpdate)

	let msDisplay = document.getElementById("ms");
	msDisplay.innerText = deltaTime * 1000 + "ms";

	frameCount = 0;
}

async function draw() {
	let now = Date.now();
	let deltaTime = (now - lastFrame) / 1000;

	updateStats(deltaTime);

	gl.clear(gl.COLOR_BUFFER_BIT);

	gl.useProgram(program);

	gl.bindBuffer(gl.ARRAY_BUFFER, vertexBuffer);

	gl.drawArrays(gl.TRIANGLES, 0, vertices.length / 2);

	lastFrame = now;
	requestAnimationFrame(draw);
}

window.addEventListener("DOMContentLoaded", init);